'use strict';

var youtube = require('./utils/youtube.js');
var path = require('path');
var url = require('url');
var jobs = require('./jobs');
var rewriteURL = require('./urls').rewriteURL;

var generateImage = function(file, targetDir, width, height) {
  return jobs.run('imageResizer', file, targetDir, width, height);
}

var paramInHrefParser = function(href) {
  var sepIndex = href.indexOf(' =');
  var ret = {
    href: href,
    options: {}
  }
  if(sepIndex < 0) {
    return ret;
  } else {
    ret.href = href.substr(0, sepIndex);
    var options = href.substr(sepIndex+2);
    options = options // split by ',' but not '\,'
      .replace('a,','\u0007')
      .split(',')
      .map(function(e) {return e.replace('\u0007','a,')});
    options.forEach(function(option) {
      var index = option.indexOf('=');
      if(index < 0) {
        ret.options[option] = true;
      } else {
        var key = option.substr(0,index);
        var value = option.substr(index+1);
        ret.options[key] = value;
      }
    });
    return ret;
  }
}

var renderYoutube = function(href, options) {
  var info = youtube.info(href);
  let out = '<div class="video-container"><div class="videodiv">';
  out += '<iframe src="https://www.youtube.com/embed/'+info.videoid+
  '?modestbranding&start='+info.time+'" frameborder="0" allowfullscreen></iframe>'
  out += '</div></div>'

  return out;
}

module.exports = function(href, title, text, curFile, cfg) {
  var parsed = paramInHrefParser(href);
  href = parsed.href;
  var options = parsed.options;

  for(let option in options) {
    if( !options.hasOwnProperty(option) ) continue;

    if(option.match(/x[0-9]+/) || option.match(/[0-9]+x[0-9]*/)) { // 123x or x123 or 123x123
      options.size = option.split('x');
      if(options.size !== undefined) {
        if(options.size[0].length > 0) {
          options.width = options.size[0];
        }
        if(options.size.length > 1 && options.size[1].length > 0) {
          options.height = options.size[1];
        }
      }
    }
  }

  if(youtube.isVideo(href)) {
    return renderYoutube(href, options);
  } else {
    var genHTML = function(file, norel) {
      var rel = file;
      if(!norel) path.relative(process.cwd(),file);

      var out = '<img src="' + rel + '" alt="' + text + '"';
      if(options.width) {
        out += ' width="' + options.width + '"';
      }
      if(options.height) {
        out += ' height="' + options.height + '"';
      }
      if (title) {
        out += ' title="' + title + '"';
      }

      out += '>';
      if(!options.nolightbox) { // if lightbox
        var a = '<a href="'+href+'"';
        a += ' data-lightbox="group"';
        a += ' data-title="' + text + '"';
        a += '>';
        out = a+out+'</a>';
      }
      return out;
    }
    var parsedHref = url.parse(href);

    if(parsedHref.host) {
      return genHTML(href, true);
    } else if(path.isAbsolute(href)) {
      return genHTML(href);
    } else {
      curFile = curFile.replace(/\.md$/,'');
      var targetDir = rewriteURL(curFile);
      targetDir = path.resolve(cfg.builddir, targetDir);
      var absSrc = path.resolve(cfg.staticDir, curFile, href);

      if(!options.size) options.size = [];
      var w = options.size[0],
          h = options.size[1];
      if(w === '') w = undefined; else w = Number(w);
      if(h === '') h = undefined; else h = Number(h);
      if(w === undefined && h === undefined) w = 512;
      var file = generateImage(absSrc, targetDir, w, h);
      return genHTML(file);
    }
  }
}

module.exports._test = {
  paramInHrefParser: paramInHrefParser
}
